(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/common'), require('@angular/core'), require('@angular/forms'), require('@ionic/angular'), require('payform'), require('rxjs')) :
    typeof define === 'function' && define.amd ? define('ion-custom-form-builder', ['exports', '@angular/common', '@angular/core', '@angular/forms', '@ionic/angular', 'payform', 'rxjs'], factory) :
    (global = global || self, factory(global['ion-custom-form-builder'] = {}, global.ng.common, global.ng.core, global.ng.forms, global.angular, global.payform, global.rxjs));
}(this, (function (exports, common, core, forms, angular, payform, rxjs) { 'use strict';

    var ION_CUSTOM_FORM_BUILDER_CONFIG = new core.InjectionToken('ION_CUSTOM_FORM_BUILDER_CONFIG');

    var IonCustomFormBuilderService = /** @class */ (function () {
        function IonCustomFormBuilderService() {
            /**
             *
             *
             * @type {Subject<boolean>}
             * @memberof IonCustomFormBuilderService
             */
            this.triggerFormSubmission$ = new rxjs.Subject();
        }
        IonCustomFormBuilderService.ɵprov = core.ɵɵdefineInjectable({ factory: function IonCustomFormBuilderService_Factory() { return new IonCustomFormBuilderService(); }, token: IonCustomFormBuilderService, providedIn: "root" });
        IonCustomFormBuilderService.decorators = [
            { type: core.Injectable, args: [{ providedIn: 'root' },] }
        ];
        IonCustomFormBuilderService.ctorParameters = function () { return []; };
        return IonCustomFormBuilderService;
    }());

    var IonCustomFormBuilderComponent = /** @class */ (function () {
        function IonCustomFormBuilderComponent(ionCustomFormBuilderService, formBuilder, config) {
            this.ionCustomFormBuilderService = ionCustomFormBuilderService;
            this.formBuilder = formBuilder;
            this.config = config;
            /**
             *
             *
             * @type {*}
             * @memberof IonCustomFormBuilderComponent
             */
            this.formControls = {};
            /**
             *
             *
             * @memberof IonCustomFormBuilderComponent
             */
            this.formBuilt = false;
            /**
             *
             *
             * @memberof IonCustomFormBuilderComponent
             */
            this.creditCardImg = 'generic.svg';
            /**
             *
             *
             * @type {FormField []}
             * @memberof IonCustomFormBuilderComponent
             */
            this.formFields = [];
            /**
             *
             *
             * @type {boolean}
             * @memberof IonCustomFormBuilderComponent
             */
            this.showSubmitButton = true;
            /**
             *
             *
             * @memberof IonCustomFormBuilderComponent
             */
            this.submitButtonText = 'Submit';
            /**
             *
             *
             * @memberof IonCustomFormBuilderComponent
             */
            this.showLabels = true;
            /**
             *
             *
             * @memberof IonCustomFormBuilderComponent
             */
            this.showIcons = true;
            /**
             *
             *
             * @memberof IonCustomFormBuilderComponent
             */
            this.showCardIcons = true;
            /**
             *
             *
             * @memberof IonCustomFormBuilderComponent
             */
            this.returnCreditCardType = false;
            /**
             * Supports ionic theme colors defined in theme/variables.css i.e 'primary', 'secondary'
             *
             * @type {string}
             * @memberof IonCustomFormBuilderComponent
             */
            this.iconColor = this.config ? this.config.defaultIconColor : undefined;
            /**
             * Supports ionic theme colors defined in theme/variables.css i.e 'primary', 'secondary'
             *
             * @type {string}
             * @memberof IonCustomFormBuilderComponent
             */
            this.submitButtonColor = this.config ? this.config.submitButtonColor : undefined;
            /**
             *
             *
             * @type {EventEmitter<any>}
             * @memberof IonCustomFormBuilderComponent
             */
            this.formSubmission = new core.EventEmitter();
        }
        IonCustomFormBuilderComponent.prototype.ngOnInit = function () {
            this.buildFormControlsConfig();
            this.watchCreditCardInput();
            this.watchFormSubmissionTriggerByService();
        };
        IonCustomFormBuilderComponent.prototype.ngOnDestroy = function () {
            if (this.creditCardInputSubscription) {
                this.creditCardInputSubscription.unsubscribe();
            }
            if (this.submissionTriggerSubscription) {
                this.submissionTriggerSubscription.unsubscribe();
            }
        };
        /**
        * Creates credit card icon css
        *
        * @param {*} icon
        * @return {*}
        * @memberof IonCustomFormBuilderComponent
        */
        IonCustomFormBuilderComponent.prototype.createCardIcon = function (icon) {
            var iconUrl = "assets/" + icon;
            return {
                'width': '14%',
                'margin-top': icon === 'master.svg' ? '10px' : '5px',
                'margin-right': '12px',
                'content': "url(" + iconUrl + ")"
            };
        };
        /**
         * Event handler for submit button clicked
         *
         * @memberof IonCustomFormBuilderComponent
         */
        IonCustomFormBuilderComponent.prototype.onSubmit = function () {
            if (this.checkErrors().length > 0) {
                return;
            }
            this.formSubmission.emit(this.customForm.value);
        };
        /**
         * Returns the abstract control for a form field by name
         *
         * @private
         * @param {number} index
         * @return {*}
         * @memberof IonCustomFormBuilderComponent
         */
        IonCustomFormBuilderComponent.prototype.getFormControlByName = function (formControlName) {
            return this.customForm.get(formControlName);
        };
        /**
         * Builds the formControls configuration
         *
         * @private
         * @memberof IonCustomFormBuilderComponent
         */
        IonCustomFormBuilderComponent.prototype.buildFormControlsConfig = function () {
            var _this = this;
            this.formFields.forEach(function (element, index) {
                switch (element.formFieldType) {
                    case 'credit-card':
                        _this.formFields[index].validationMessages.push({
                            type: 'creditCardValidator',
                            message: 'Credit card number is invalid'
                        });
                        _this.creditCardFieldIndex = index;
                        _this.formControls["" + element.formControlName] = new forms.FormControl(null, {
                            validators: element.validators,
                            asyncValidators: [
                                _this.creditCardValidator.bind(_this)
                            ]
                        });
                        break;
                    default:
                        if (element.asyncValidators) {
                            _this.formControls["" + element.formControlName] = new forms.FormControl(null, {
                                validators: element.validators,
                                asyncValidators: _this.processAsyncValidators(element, _this)
                            });
                        }
                        else {
                            _this.formControls["" + element.formControlName] = ['', element.validators];
                        }
                        break;
                }
            });
            this.customForm = this.formBuilder.group(this.formControls);
            this.prePopulateForm();
            this.formBuilt = true;
        };
        /**
         * Binds abstractControls to asyncValidator functions
         *
         * @private
         * @param {FormField} element
         * @param {this} form
         * @return {*}
         * @memberof IonCustomFormBuilderComponent
         */
        IonCustomFormBuilderComponent.prototype.processAsyncValidators = function (element, abstractControl) {
            element.asyncValidators.forEach(function (asyncValidator) {
                asyncValidator.bind(abstractControl);
            });
            return element.asyncValidators;
        };
        /**
         * Creates Subscription to watch form submission triggered by service
         *
         * @private
         * @memberof IonCustomFormBuilderComponent
         */
        IonCustomFormBuilderComponent.prototype.watchFormSubmissionTriggerByService = function () {
            var _this = this;
            this.submissionTriggerSubscription = this.ionCustomFormBuilderService.triggerFormSubmission$.subscribe(function (trigger) {
                if (trigger) {
                    _this.onSubmit();
                }
            });
        };
        /**
         * Creates a subscription to watch credit-card input valueChanges
         *
         * @private
         * @memberof IonCustomFormBuilderComponent
         */
        IonCustomFormBuilderComponent.prototype.watchCreditCardInput = function () {
            var _this = this;
            if (this.creditCardFieldIndex !== undefined) {
                this.creditCardInputSubscription = this.getFormControlByIndex(this.creditCardFieldIndex)
                    .valueChanges
                    .subscribe(function (cardNumberValue) {
                    _this.detectCardType(cardNumberValue);
                });
            }
        };
        /**
         * Validates the credit card number
         *
         * @private
         * @param {AbstractControl} control
         * @return {*}  {Promise<any>}
         * @memberof IonCustomFormBuilderComponent
         */
        IonCustomFormBuilderComponent.prototype.creditCardValidator = function (control) {
            if (!control.parent) {
                return Promise.resolve(null);
            }
            else if (control !== undefined && control.value && !this.isCardNumberValid(control.value)) {
                control.markAsTouched({ onlySelf: true });
                return Promise.resolve({ creditCardValidator: { valid: false } });
            }
            else {
                return Promise.resolve(null);
            }
        };
        /**
         * Pre-populates the form with initial values if provided
         *
         * @private
         * @memberof IonCustomFormBuilderComponent
         */
        IonCustomFormBuilderComponent.prototype.prePopulateForm = function () {
            var _this = this;
            this.formFields.forEach(function (element, index, self) {
                if (element.value !== undefined) {
                    _this.getFormControlByIndex(index).setValue(element.value);
                    _this.getFormControlByIndex(index).markAsTouched();
                }
            });
        };
        /**
         * Returns the abstract control for a form field by index
         *
         * @private
         * @param {number} index
         * @return {*}
         * @memberof IonCustomFormBuilderComponent
         */
        IonCustomFormBuilderComponent.prototype.getFormControlByIndex = function (index) {
            return this.customForm.get("" + this.formFields[index].formControlName);
        };
        /**
         * Validates card number using payform lib
         *
         * @private
         * @param {*} cardNumber
         * @memberof IonCustomFormBuilderComponent
         */
        IonCustomFormBuilderComponent.prototype.isCardNumberValid = function (cardNumber) {
            return payform.validateCardNumber(cardNumber);
        };
        /**
         * Detects card type using payform lib
         *
         * @private
         * @param {*} cardNumber
         * @memberof IonCustomFormBuilderComponent
         */
        IonCustomFormBuilderComponent.prototype.detectCardType = function (cardNumber) {
            if (payform.parseCardType(cardNumber) === 'visa') {
                this.creditCardImg = 'visa.svg';
            }
            else if (payform.parseCardType(cardNumber) === 'amex') {
                this.creditCardImg = 'amex.svg';
            }
            else if (payform.parseCardType(cardNumber) === 'mastercard') {
                this.creditCardImg = 'master.svg';
            }
            else {
                this.creditCardImg = 'generic.svg';
            }
        };
        /**
         * Checks if form has errors
         *
         * @private
         * @return {*}
         * @memberof IonCustomFormBuilderComponent
         */
        IonCustomFormBuilderComponent.prototype.checkErrors = function () {
            var _this = this;
            var errors = [];
            Object.keys(this.customForm.controls).forEach(function (field) {
                var formControlErrors = _this.customForm.get(field).errors;
                if (formControlErrors != null) {
                    errors.push({
                        control: field,
                        errors: formControlErrors
                    });
                    var control = _this.customForm.get(field);
                    control.markAsTouched({ onlySelf: true });
                }
            });
            return errors;
        };
        IonCustomFormBuilderComponent.ctorParameters = function () { return [
            { type: IonCustomFormBuilderService },
            { type: forms.FormBuilder },
            { type: undefined, decorators: [{ type: core.Inject, args: [ION_CUSTOM_FORM_BUILDER_CONFIG,] }] }
        ]; };
        IonCustomFormBuilderComponent.decorators = [
            { type: core.Component, args: [{
                        selector: 'ion-custom-form-builder',
                        template: "<form [formGroup]=\"customForm\" novalidate *ngIf=\"formBuilt\" (ngSubmit)=\"onSubmit()\">\n  <ng-container *ngFor=\"let field of formFields\">\n    <ion-row class=\"ion-cfb-row\">\n      <ion-col class=\"ion-cfb-col\">\n        <!-- ion-input Text field controller -->\n        <ng-container *ngIf=\"field.formFieldType === 'ion-input' || field.formFieldType === undefined\">\n          <ion-item class=\"ion-cfb-input-item\">\n            <ion-icon class=\"ion-cfb-input-icon\" slot=\"start\" name=\"{{ field.icon }}\" color=\"{{iconColor ? iconColor : 'primary'}}\" *ngIf=\"showIcons\"></ion-icon>\n            <ion-label class=\"ion-cfb-input-label\" position=\"{{field?.labelPosition ? field?.labelPosition : 'floating'}}\" *ngIf=\"showLabels\">{{\n              field.title }}\n            </ion-label>\n            <ion-input class=\"ion-cfb-ion-input\" *ngIf=\"field?.placeholder\"  placeholder=\"{{field?.placeholder}}\" type=\"{{ field.type }}\"\n              formControlName=\"{{ field.formControlName }}\">\n            </ion-input>\n            <ion-input class=\"ion-cfb-ion-input\" *ngIf=\"field?.placeholder === undefined\"  type=\"{{ field.type }}\"\n              formControlName=\"{{ field.formControlName }}\">\n            </ion-input>\n          </ion-item>\n          <div class=\"ion-cfb-validation-messages\" *ngIf=\"field?.validationMessages\">\n            <ng-container *ngFor=\"let validation of field?.validationMessages\">\n              <div *ngIf=\"getFormControlByName(field.formControlName).hasError(validation.type) &&(getFormControlByName(field.formControlName).dirty || getFormControlByName(field.formControlName).touched)\">\n                <ion-text color=\"danger\"><small>{{ validation.message }}</small></ion-text>\n              </div>\n            </ng-container>\n          </div>\n        </ng-container>\n\n\n        <!-- Credit card field controller -->\n        <ng-container *ngIf=\"field.formFieldType === 'credit-card'\">\n          <ion-item class=\"ion-cfb-input-item\">\n            <div *ngIf=\"showCardIcons\" slot=\"start\" [ngStyle]=\"createCardIcon(creditCardImg)\"></div>\n            <ion-label class=\"ion-cfb-input-label\" position=\"{{field?.labelPosition ? field?.labelPosition : 'floating'}}\" *ngIf=\"showLabels\">{{\n              field.title }}\n            </ion-label>\n            <ion-input class=\"ion-cfb-ion-input\" *ngIf=\"field?.placeholder\"  placeholder=\"{{field?.placeholder}}\" type=\"{{ field.type }}\"\n              formControlName=\"{{ field.formControlName }}\">\n            </ion-input>\n            <ion-input class=\"ion-cfb-ion-input\" *ngIf=\"field?.placeholder === undefined\"  type=\"{{ field.type }}\"\n              formControlName=\"{{ field.formControlName }}\">\n            </ion-input>\n          </ion-item>\n          <div class=\"ion-cfb-validation-messages\" *ngIf=\"field?.validationMessages\">\n            <ng-container *ngFor=\"let validation of field?.validationMessages\">\n              <div *ngIf=\"getFormControlByName(field.formControlName).hasError(validation.type) &&(getFormControlByName(field.formControlName).dirty || getFormControlByName(field.formControlName).touched)\">\n                <ion-text color=\"danger\"><small>{{ validation.message }}</small></ion-text>\n              </div>\n            </ng-container>\n          </div>\n        </ng-container>\n\n        <!-- Text Area Controller -->\n        <ng-container *ngIf=\"field.formFieldType === 'ion-textarea'\">\n          <ion-item class=\"ion-cfb-textarea-item\">\n            <ion-label class=\"ion-cfb-textarea-label\" position=\"{{field?.labelPosition ? field?.labelPosition : 'floating'}}\" *ngIf=\"showLabels\">{{\n              field.title }}\n            </ion-label>\n            <ion-textarea class=\"ion-cfb-ion-textarea\" *ngIf=\"field?.placeholder\"  rows=\"{{ field?.textAreaRowCount }}\"\n              formControlName=\"{{ field.formControlName }}\" placeholder=\"{{field?.placeholder}}\"></ion-textarea>\n            <ion-textarea class=\"ion-cfb-ion-textarea\" *ngIf=\"field?.placeholder === undefined\"  rows=\"{{ field?.textAreaRowCount }}\"\n              formControlName=\"{{ field.formControlName }}\">\n            </ion-textarea>\n          </ion-item>\n          <div class=\"ion-cfb-validation-messages\" *ngIf=\"field?.validationMessages\">\n            <ng-container *ngFor=\"let validation of field?.validationMessages\">\n              <div *ngIf=\"getFormControlByName(field.formControlName).hasError(validation.type) &&(getFormControlByName(field.formControlName).dirty || getFormControlByName(field.formControlName).touched)\">\n                <ion-text color=\"danger\"><small>{{ validation.message }}</small></ion-text>\n              </div>\n            </ng-container>\n          </div>\n        </ng-container>\n      </ion-col>\n    </ion-row>\n  </ng-container>\n\n\n  <!-- Submit Button -->\n  <ion-row *ngIf=\"showSubmitButton\" class=\"ion-cfb-submit-btn-row\">\n    <ion-col class=\"ion-cfb-submit-btn-col\">\n      <ion-button  class=\"ion-cfb-submit-btn\" expand=\"block\" color=\"{{submitButtonColor ? submitButtonColor : 'primary'}}\" type=\"submit\">\n      {{ submitButtonText }}\n      </ion-button>\n    </ion-col>\n  </ion-row>\n</form>\n",
                        styles: [".ion-cfb-validation-messages{margin-left:16px}"]
                    },] }
        ];
        IonCustomFormBuilderComponent.ctorParameters = function () { return [
            { type: IonCustomFormBuilderService },
            { type: forms.FormBuilder },
            { type: undefined, decorators: [{ type: core.Inject, args: [ION_CUSTOM_FORM_BUILDER_CONFIG,] }] }
        ]; };
        IonCustomFormBuilderComponent.propDecorators = {
            formFields: [{ type: core.Input }],
            showSubmitButton: [{ type: core.Input }],
            submitButtonText: [{ type: core.Input }],
            showLabels: [{ type: core.Input }],
            showIcons: [{ type: core.Input }],
            showCardIcons: [{ type: core.Input }],
            returnCreditCardType: [{ type: core.Input }],
            iconColor: [{ type: core.Input }],
            submitButtonColor: [{ type: core.Input }],
            formSubmission: [{ type: core.Output }]
        };
        return IonCustomFormBuilderComponent;
    }());

    var Components = [
        IonCustomFormBuilderComponent
    ];
    var IonCustomFormBuilderModule = /** @class */ (function () {
        function IonCustomFormBuilderModule() {
        }
        IonCustomFormBuilderModule.forRoot = function (options) {
            return {
                ngModule: IonCustomFormBuilderModule,
                providers: [
                    {
                        provide: ION_CUSTOM_FORM_BUILDER_CONFIG,
                        useValue: options
                    }
                ]
            };
        };
        IonCustomFormBuilderModule.decorators = [
            { type: core.NgModule, args: [{
                        declarations: [
                            Components
                        ],
                        imports: [
                            angular.IonicModule,
                            common.CommonModule,
                            forms.ReactiveFormsModule,
                            forms.FormsModule,
                        ],
                        exports: [
                            Components
                        ]
                    },] }
        ];
        return IonCustomFormBuilderModule;
    }());

    exports.IonCustomFormBuilderModule = IonCustomFormBuilderModule;
    exports.IonCustomFormBuilderService = IonCustomFormBuilderService;
    exports.ɵa = IonCustomFormBuilderComponent;
    exports.ɵb = ION_CUSTOM_FORM_BUILDER_CONFIG;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ion-custom-form-builder.umd.js.map
