import { CommonModule } from '@angular/common';
import { InjectionToken, ɵɵdefineInjectable, Injectable, EventEmitter, Inject, Component, Input, Output, NgModule } from '@angular/core';
import { FormControl, FormBuilder, ReactiveFormsModule, FormsModule } from '@angular/forms';
import { IonicModule } from '@ionic/angular';
import { validateCardNumber, parseCardType } from 'payform';
import { Subject } from 'rxjs';

const ION_CUSTOM_FORM_BUILDER_CONFIG = new InjectionToken('ION_CUSTOM_FORM_BUILDER_CONFIG');

class IonCustomFormBuilderService {
    constructor() {
        /**
         *
         *
         * @type {Subject<boolean>}
         * @memberof IonCustomFormBuilderService
         */
        this.triggerFormSubmission$ = new Subject();
    }
}
IonCustomFormBuilderService.ɵprov = ɵɵdefineInjectable({ factory: function IonCustomFormBuilderService_Factory() { return new IonCustomFormBuilderService(); }, token: IonCustomFormBuilderService, providedIn: "root" });
IonCustomFormBuilderService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
IonCustomFormBuilderService.ctorParameters = () => [];

class IonCustomFormBuilderComponent {
    constructor(ionCustomFormBuilderService, formBuilder, config) {
        this.ionCustomFormBuilderService = ionCustomFormBuilderService;
        this.formBuilder = formBuilder;
        this.config = config;
        /**
         *
         *
         * @type {*}
         * @memberof IonCustomFormBuilderComponent
         */
        this.formControls = {};
        /**
         *
         *
         * @memberof IonCustomFormBuilderComponent
         */
        this.formBuilt = false;
        /**
         *
         *
         * @memberof IonCustomFormBuilderComponent
         */
        this.creditCardImg = 'generic.svg';
        /**
         *
         *
         * @type {FormField []}
         * @memberof IonCustomFormBuilderComponent
         */
        this.formFields = [];
        /**
         *
         *
         * @type {boolean}
         * @memberof IonCustomFormBuilderComponent
         */
        this.showSubmitButton = true;
        /**
         *
         *
         * @memberof IonCustomFormBuilderComponent
         */
        this.submitButtonText = 'Submit';
        /**
         *
         *
         * @memberof IonCustomFormBuilderComponent
         */
        this.showLabels = true;
        /**
         *
         *
         * @memberof IonCustomFormBuilderComponent
         */
        this.showIcons = true;
        /**
         *
         *
         * @memberof IonCustomFormBuilderComponent
         */
        this.showCardIcons = true;
        /**
         *
         *
         * @memberof IonCustomFormBuilderComponent
         */
        this.returnCreditCardType = false;
        /**
         * Supports ionic theme colors defined in theme/variables.css i.e 'primary', 'secondary'
         *
         * @type {string}
         * @memberof IonCustomFormBuilderComponent
         */
        this.iconColor = this.config ? this.config.defaultIconColor : undefined;
        /**
         * Supports ionic theme colors defined in theme/variables.css i.e 'primary', 'secondary'
         *
         * @type {string}
         * @memberof IonCustomFormBuilderComponent
         */
        this.submitButtonColor = this.config ? this.config.submitButtonColor : undefined;
        /**
         *
         *
         * @type {EventEmitter<any>}
         * @memberof IonCustomFormBuilderComponent
         */
        this.formSubmission = new EventEmitter();
    }
    ngOnInit() {
        this.buildFormControlsConfig();
        this.watchCreditCardInput();
        this.watchFormSubmissionTriggerByService();
    }
    ngOnDestroy() {
        if (this.creditCardInputSubscription) {
            this.creditCardInputSubscription.unsubscribe();
        }
        if (this.submissionTriggerSubscription) {
            this.submissionTriggerSubscription.unsubscribe();
        }
    }
    /**
    * Creates credit card icon css
    *
    * @param {*} icon
    * @return {*}
    * @memberof IonCustomFormBuilderComponent
    */
    createCardIcon(icon) {
        const iconUrl = `assets/${icon}`;
        return {
            'width': '14%',
            'margin-top': icon === 'master.svg' ? '10px' : '5px',
            'margin-right': '12px',
            'content': `url(${iconUrl})`
        };
    }
    /**
     * Event handler for submit button clicked
     *
     * @memberof IonCustomFormBuilderComponent
     */
    onSubmit() {
        if (this.checkErrors().length > 0) {
            return;
        }
        this.formSubmission.emit(this.customForm.value);
    }
    /**
     * Returns the abstract control for a form field by name
     *
     * @private
     * @param {number} index
     * @return {*}
     * @memberof IonCustomFormBuilderComponent
     */
    getFormControlByName(formControlName) {
        return this.customForm.get(formControlName);
    }
    /**
     * Builds the formControls configuration
     *
     * @private
     * @memberof IonCustomFormBuilderComponent
     */
    buildFormControlsConfig() {
        this.formFields.forEach((element, index) => {
            switch (element.formFieldType) {
                case 'credit-card':
                    this.formFields[index].validationMessages.push({
                        type: 'creditCardValidator',
                        message: 'Credit card number is invalid'
                    });
                    this.creditCardFieldIndex = index;
                    this.formControls[`${element.formControlName}`] = new FormControl(null, {
                        validators: element.validators,
                        asyncValidators: [
                            this.creditCardValidator.bind(this)
                        ]
                    });
                    break;
                default:
                    if (element.asyncValidators) {
                        this.formControls[`${element.formControlName}`] = new FormControl(null, {
                            validators: element.validators,
                            asyncValidators: this.processAsyncValidators(element, this)
                        });
                    }
                    else {
                        this.formControls[`${element.formControlName}`] = ['', element.validators];
                    }
                    break;
            }
        });
        this.customForm = this.formBuilder.group(this.formControls);
        this.prePopulateForm();
        this.formBuilt = true;
    }
    /**
     * Binds abstractControls to asyncValidator functions
     *
     * @private
     * @param {FormField} element
     * @param {this} form
     * @return {*}
     * @memberof IonCustomFormBuilderComponent
     */
    processAsyncValidators(element, abstractControl) {
        element.asyncValidators.forEach(asyncValidator => {
            asyncValidator.bind(abstractControl);
        });
        return element.asyncValidators;
    }
    /**
     * Creates Subscription to watch form submission triggered by service
     *
     * @private
     * @memberof IonCustomFormBuilderComponent
     */
    watchFormSubmissionTriggerByService() {
        this.submissionTriggerSubscription = this.ionCustomFormBuilderService.triggerFormSubmission$.subscribe(trigger => {
            if (trigger) {
                this.onSubmit();
            }
        });
    }
    /**
     * Creates a subscription to watch credit-card input valueChanges
     *
     * @private
     * @memberof IonCustomFormBuilderComponent
     */
    watchCreditCardInput() {
        if (this.creditCardFieldIndex !== undefined) {
            this.creditCardInputSubscription = this.getFormControlByIndex(this.creditCardFieldIndex)
                .valueChanges
                .subscribe(cardNumberValue => {
                this.detectCardType(cardNumberValue);
            });
        }
    }
    /**
     * Validates the credit card number
     *
     * @private
     * @param {AbstractControl} control
     * @return {*}  {Promise<any>}
     * @memberof IonCustomFormBuilderComponent
     */
    creditCardValidator(control) {
        if (!control.parent) {
            return Promise.resolve(null);
        }
        else if (control !== undefined && control.value && !this.isCardNumberValid(control.value)) {
            control.markAsTouched({ onlySelf: true });
            return Promise.resolve({ creditCardValidator: { valid: false } });
        }
        else {
            return Promise.resolve(null);
        }
    }
    /**
     * Pre-populates the form with initial values if provided
     *
     * @private
     * @memberof IonCustomFormBuilderComponent
     */
    prePopulateForm() {
        this.formFields.forEach((element, index, self) => {
            if (element.value !== undefined) {
                this.getFormControlByIndex(index).setValue(element.value);
                this.getFormControlByIndex(index).markAsTouched();
            }
        });
    }
    /**
     * Returns the abstract control for a form field by index
     *
     * @private
     * @param {number} index
     * @return {*}
     * @memberof IonCustomFormBuilderComponent
     */
    getFormControlByIndex(index) {
        return this.customForm.get(`${this.formFields[index].formControlName}`);
    }
    /**
     * Validates card number using payform lib
     *
     * @private
     * @param {*} cardNumber
     * @memberof IonCustomFormBuilderComponent
     */
    isCardNumberValid(cardNumber) {
        return validateCardNumber(cardNumber);
    }
    /**
     * Detects card type using payform lib
     *
     * @private
     * @param {*} cardNumber
     * @memberof IonCustomFormBuilderComponent
     */
    detectCardType(cardNumber) {
        if (parseCardType(cardNumber) === 'visa') {
            this.creditCardImg = 'visa.svg';
        }
        else if (parseCardType(cardNumber) === 'amex') {
            this.creditCardImg = 'amex.svg';
        }
        else if (parseCardType(cardNumber) === 'mastercard') {
            this.creditCardImg = 'master.svg';
        }
        else {
            this.creditCardImg = 'generic.svg';
        }
    }
    /**
     * Checks if form has errors
     *
     * @private
     * @return {*}
     * @memberof IonCustomFormBuilderComponent
     */
    checkErrors() {
        let errors = [];
        Object.keys(this.customForm.controls).forEach(field => {
            const formControlErrors = this.customForm.get(field).errors;
            if (formControlErrors != null) {
                errors.push({
                    control: field,
                    errors: formControlErrors
                });
                const control = this.customForm.get(field);
                control.markAsTouched({ onlySelf: true });
            }
        });
        return errors;
    }
}
IonCustomFormBuilderComponent.ctorParameters = () => [
    { type: IonCustomFormBuilderService },
    { type: FormBuilder },
    { type: undefined, decorators: [{ type: Inject, args: [ION_CUSTOM_FORM_BUILDER_CONFIG,] }] }
];
IonCustomFormBuilderComponent.decorators = [
    { type: Component, args: [{
                selector: 'ion-custom-form-builder',
                template: "<form [formGroup]=\"customForm\" novalidate *ngIf=\"formBuilt\" (ngSubmit)=\"onSubmit()\">\n  <ng-container *ngFor=\"let field of formFields\">\n    <ion-row class=\"ion-cfb-row\">\n      <ion-col class=\"ion-cfb-col\">\n        <!-- ion-input Text field controller -->\n        <ng-container *ngIf=\"field.formFieldType === 'ion-input' || field.formFieldType === undefined\">\n          <ion-item class=\"ion-cfb-input-item\">\n            <ion-icon class=\"ion-cfb-input-icon\" slot=\"start\" name=\"{{ field.icon }}\" color=\"{{iconColor ? iconColor : 'primary'}}\" *ngIf=\"showIcons\"></ion-icon>\n            <ion-label class=\"ion-cfb-input-label\" position=\"{{field?.labelPosition ? field?.labelPosition : 'floating'}}\" *ngIf=\"showLabels\">{{\n              field.title }}\n            </ion-label>\n            <ion-input class=\"ion-cfb-ion-input\" *ngIf=\"field?.placeholder\"  placeholder=\"{{field?.placeholder}}\" type=\"{{ field.type }}\"\n              formControlName=\"{{ field.formControlName }}\">\n            </ion-input>\n            <ion-input class=\"ion-cfb-ion-input\" *ngIf=\"field?.placeholder === undefined\"  type=\"{{ field.type }}\"\n              formControlName=\"{{ field.formControlName }}\">\n            </ion-input>\n          </ion-item>\n          <div class=\"ion-cfb-validation-messages\" *ngIf=\"field?.validationMessages\">\n            <ng-container *ngFor=\"let validation of field?.validationMessages\">\n              <div *ngIf=\"getFormControlByName(field.formControlName).hasError(validation.type) &&(getFormControlByName(field.formControlName).dirty || getFormControlByName(field.formControlName).touched)\">\n                <ion-text color=\"danger\"><small>{{ validation.message }}</small></ion-text>\n              </div>\n            </ng-container>\n          </div>\n        </ng-container>\n\n\n        <!-- Credit card field controller -->\n        <ng-container *ngIf=\"field.formFieldType === 'credit-card'\">\n          <ion-item class=\"ion-cfb-input-item\">\n            <div *ngIf=\"showCardIcons\" slot=\"start\" [ngStyle]=\"createCardIcon(creditCardImg)\"></div>\n            <ion-label class=\"ion-cfb-input-label\" position=\"{{field?.labelPosition ? field?.labelPosition : 'floating'}}\" *ngIf=\"showLabels\">{{\n              field.title }}\n            </ion-label>\n            <ion-input class=\"ion-cfb-ion-input\" *ngIf=\"field?.placeholder\"  placeholder=\"{{field?.placeholder}}\" type=\"{{ field.type }}\"\n              formControlName=\"{{ field.formControlName }}\">\n            </ion-input>\n            <ion-input class=\"ion-cfb-ion-input\" *ngIf=\"field?.placeholder === undefined\"  type=\"{{ field.type }}\"\n              formControlName=\"{{ field.formControlName }}\">\n            </ion-input>\n          </ion-item>\n          <div class=\"ion-cfb-validation-messages\" *ngIf=\"field?.validationMessages\">\n            <ng-container *ngFor=\"let validation of field?.validationMessages\">\n              <div *ngIf=\"getFormControlByName(field.formControlName).hasError(validation.type) &&(getFormControlByName(field.formControlName).dirty || getFormControlByName(field.formControlName).touched)\">\n                <ion-text color=\"danger\"><small>{{ validation.message }}</small></ion-text>\n              </div>\n            </ng-container>\n          </div>\n        </ng-container>\n\n        <!-- Text Area Controller -->\n        <ng-container *ngIf=\"field.formFieldType === 'ion-textarea'\">\n          <ion-item class=\"ion-cfb-textarea-item\">\n            <ion-label class=\"ion-cfb-textarea-label\" position=\"{{field?.labelPosition ? field?.labelPosition : 'floating'}}\" *ngIf=\"showLabels\">{{\n              field.title }}\n            </ion-label>\n            <ion-textarea class=\"ion-cfb-ion-textarea\" *ngIf=\"field?.placeholder\"  rows=\"{{ field?.textAreaRowCount }}\"\n              formControlName=\"{{ field.formControlName }}\" placeholder=\"{{field?.placeholder}}\"></ion-textarea>\n            <ion-textarea class=\"ion-cfb-ion-textarea\" *ngIf=\"field?.placeholder === undefined\"  rows=\"{{ field?.textAreaRowCount }}\"\n              formControlName=\"{{ field.formControlName }}\">\n            </ion-textarea>\n          </ion-item>\n          <div class=\"ion-cfb-validation-messages\" *ngIf=\"field?.validationMessages\">\n            <ng-container *ngFor=\"let validation of field?.validationMessages\">\n              <div *ngIf=\"getFormControlByName(field.formControlName).hasError(validation.type) &&(getFormControlByName(field.formControlName).dirty || getFormControlByName(field.formControlName).touched)\">\n                <ion-text color=\"danger\"><small>{{ validation.message }}</small></ion-text>\n              </div>\n            </ng-container>\n          </div>\n        </ng-container>\n      </ion-col>\n    </ion-row>\n  </ng-container>\n\n\n  <!-- Submit Button -->\n  <ion-row *ngIf=\"showSubmitButton\" class=\"ion-cfb-submit-btn-row\">\n    <ion-col class=\"ion-cfb-submit-btn-col\">\n      <ion-button  class=\"ion-cfb-submit-btn\" expand=\"block\" color=\"{{submitButtonColor ? submitButtonColor : 'primary'}}\" type=\"submit\">\n      {{ submitButtonText }}\n      </ion-button>\n    </ion-col>\n  </ion-row>\n</form>\n",
                styles: [".ion-cfb-validation-messages{margin-left:16px}"]
            },] }
];
IonCustomFormBuilderComponent.ctorParameters = () => [
    { type: IonCustomFormBuilderService },
    { type: FormBuilder },
    { type: undefined, decorators: [{ type: Inject, args: [ION_CUSTOM_FORM_BUILDER_CONFIG,] }] }
];
IonCustomFormBuilderComponent.propDecorators = {
    formFields: [{ type: Input }],
    showSubmitButton: [{ type: Input }],
    submitButtonText: [{ type: Input }],
    showLabels: [{ type: Input }],
    showIcons: [{ type: Input }],
    showCardIcons: [{ type: Input }],
    returnCreditCardType: [{ type: Input }],
    iconColor: [{ type: Input }],
    submitButtonColor: [{ type: Input }],
    formSubmission: [{ type: Output }]
};

const Components = [
    IonCustomFormBuilderComponent
];
class IonCustomFormBuilderModule {
    static forRoot(options) {
        return {
            ngModule: IonCustomFormBuilderModule,
            providers: [
                {
                    provide: ION_CUSTOM_FORM_BUILDER_CONFIG,
                    useValue: options
                }
            ]
        };
    }
}
IonCustomFormBuilderModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    Components
                ],
                imports: [
                    IonicModule,
                    CommonModule,
                    ReactiveFormsModule,
                    FormsModule,
                ],
                exports: [
                    Components
                ]
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { IonCustomFormBuilderModule, IonCustomFormBuilderService, IonCustomFormBuilderComponent as ɵa, ION_CUSTOM_FORM_BUILDER_CONFIG as ɵb };
//# sourceMappingURL=ion-custom-form-builder.js.map
