import { EventEmitter, OnDestroy, OnInit } from '@angular/core';
import { AbstractControl, FormBuilder, FormGroup } from '@angular/forms';
import { Subscription } from 'rxjs';
import { IonCustomFormBuilderConfig } from './config-options-interface';
import { FormField } from './form-field-interface';
import { IonCustomFormBuilderService } from './ion-custom-form-builder.service';
export declare class IonCustomFormBuilderComponent implements OnInit, OnDestroy {
    private ionCustomFormBuilderService;
    private formBuilder;
    private config;
    /**
     *
     *
     * @type {Subscription}
     * @memberof IonCustomFormBuilderComponent
     */
    creditCardInputSubscription: Subscription;
    /**
     *
     *
     * @type {Subscription}
     * @memberof IonCustomFormBuilderComponent
     */
    submissionTriggerSubscription: Subscription;
    /**
     *
     *
     * @type {*}
     * @memberof IonCustomFormBuilderComponent
     */
    formControls: any;
    /**
     *
     *
     * @memberof IonCustomFormBuilderComponent
     */
    formBuilt: boolean;
    /**
     *
     *
     * @type {number}
     * @memberof IonCustomFormBuilderComponent
     */
    creditCardFieldIndex: number;
    /**
     *
     *
     * @memberof IonCustomFormBuilderComponent
     */
    creditCardImg: string;
    /**
     *
     *
     * @type {FormGroup}
     * @memberof IonCustomFormBuilderComponent
     */
    customForm: FormGroup;
    /**
     *
     *
     * @type {FormField []}
     * @memberof IonCustomFormBuilderComponent
     */
    formFields: FormField[];
    /**
     *
     *
     * @type {boolean}
     * @memberof IonCustomFormBuilderComponent
     */
    showSubmitButton: boolean;
    /**
     *
     *
     * @memberof IonCustomFormBuilderComponent
     */
    submitButtonText: string;
    /**
     *
     *
     * @memberof IonCustomFormBuilderComponent
     */
    showLabels: boolean;
    /**
     *
     *
     * @memberof IonCustomFormBuilderComponent
     */
    showIcons: boolean;
    /**
     *
     *
     * @memberof IonCustomFormBuilderComponent
     */
    showCardIcons: boolean;
    /**
     *
     *
     * @memberof IonCustomFormBuilderComponent
     */
    returnCreditCardType: boolean;
    /**
     * Supports ionic theme colors defined in theme/variables.css i.e 'primary', 'secondary'
     *
     * @type {string}
     * @memberof IonCustomFormBuilderComponent
     */
    iconColor: string;
    /**
     * Supports ionic theme colors defined in theme/variables.css i.e 'primary', 'secondary'
     *
     * @type {string}
     * @memberof IonCustomFormBuilderComponent
     */
    submitButtonColor: string;
    /**
     *
     *
     * @type {EventEmitter<any>}
     * @memberof IonCustomFormBuilderComponent
     */
    formSubmission: EventEmitter<any>;
    constructor(ionCustomFormBuilderService: IonCustomFormBuilderService, formBuilder: FormBuilder, config: IonCustomFormBuilderConfig);
    ngOnInit(): void;
    ngOnDestroy(): void;
    /**
    * Creates credit card icon css
    *
    * @param {*} icon
    * @return {*}
    * @memberof IonCustomFormBuilderComponent
    */
    createCardIcon(icon: any): {
        width: string;
        'margin-top': string;
        'margin-right': string;
        content: string;
    };
    /**
     * Event handler for submit button clicked
     *
     * @memberof IonCustomFormBuilderComponent
     */
    onSubmit(): void;
    /**
     * Returns the abstract control for a form field by name
     *
     * @private
     * @param {number} index
     * @return {*}
     * @memberof IonCustomFormBuilderComponent
     */
    getFormControlByName(formControlName: string): AbstractControl;
    /**
     * Builds the formControls configuration
     *
     * @private
     * @memberof IonCustomFormBuilderComponent
     */
    private buildFormControlsConfig;
    /**
     * Binds abstractControls to asyncValidator functions
     *
     * @private
     * @param {FormField} element
     * @param {this} form
     * @return {*}
     * @memberof IonCustomFormBuilderComponent
     */
    private processAsyncValidators;
    /**
     * Creates Subscription to watch form submission triggered by service
     *
     * @private
     * @memberof IonCustomFormBuilderComponent
     */
    private watchFormSubmissionTriggerByService;
    /**
     * Creates a subscription to watch credit-card input valueChanges
     *
     * @private
     * @memberof IonCustomFormBuilderComponent
     */
    private watchCreditCardInput;
    /**
     * Validates the credit card number
     *
     * @private
     * @param {AbstractControl} control
     * @return {*}  {Promise<any>}
     * @memberof IonCustomFormBuilderComponent
     */
    private creditCardValidator;
    /**
     * Pre-populates the form with initial values if provided
     *
     * @private
     * @memberof IonCustomFormBuilderComponent
     */
    private prePopulateForm;
    /**
     * Returns the abstract control for a form field by index
     *
     * @private
     * @param {number} index
     * @return {*}
     * @memberof IonCustomFormBuilderComponent
     */
    private getFormControlByIndex;
    /**
     * Validates card number using payform lib
     *
     * @private
     * @param {*} cardNumber
     * @memberof IonCustomFormBuilderComponent
     */
    private isCardNumberValid;
    /**
     * Detects card type using payform lib
     *
     * @private
     * @param {*} cardNumber
     * @memberof IonCustomFormBuilderComponent
     */
    private detectCardType;
    /**
     * Checks if form has errors
     *
     * @private
     * @return {*}
     * @memberof IonCustomFormBuilderComponent
     */
    private checkErrors;
}
